#pragma once

using namespace System;
using namespace System::ComponentModel;
using namespace System::Collections;
using namespace System::Windows::Forms;
using namespace System::Data;
using namespace System::Drawing;
using namespace System::Drawing::Imaging;

//#include "imagefilter.h"

namespace FilterTemplate {

	public ref class SettingsForm : public System::Windows::Forms::Form
	{

	public:
		filtersettings* FilterSetting;
		bool FilterCancelled;
		bool MouseButtonDown;

		Bitmap^ PreviewBitmap;
		Rectangle SourceRect, DestinationRect;
		int SWidth, SHeight;
		int LastX, LastY, init;

		System::Drawing::Imaging::PixelFormat pFormat;


	private: System::Windows::Forms::HScrollBar^  BrightnessScrollBar;
	private: System::Windows::Forms::HScrollBar^  ContrastScrollBar;
	private: System::Windows::Forms::HScrollBar^  GammaScrollBar;
	private: System::Windows::Forms::Label^  label1;
	private: System::Windows::Forms::Label^  label2;
	private: System::Windows::Forms::Label^  label3;
	private: System::Windows::Forms::Label^  label4;
	private: System::Windows::Forms::Label^  BrightnessLabel;
	private: System::Windows::Forms::Label^  ContrastLabel;
	private: System::Windows::Forms::Label^  GammaLabel;
	private: System::Windows::Forms::CheckBox^  CheckBoxBrightnessR;
	private: System::Windows::Forms::CheckBox^  CheckBoxBrightnessG;
	private: System::Windows::Forms::CheckBox^  CheckBoxBrightnessB;
	private: System::Windows::Forms::CheckBox^  CheckBoxContrastR;
	private: System::Windows::Forms::CheckBox^  CheckBoxContrastG;
	private: System::Windows::Forms::CheckBox^  CheckBoxContrastB;
	private: System::Windows::Forms::CheckBox^  CheckBoxGammaR;
	private: System::Windows::Forms::CheckBox^  CheckBoxGammaG;
	private: System::Windows::Forms::CheckBox^  CheckBoxGammaB;




	public:
		SettingsForm(int width, int height, filtersettings* fs)
		{
			init = 1;

			InitializeComponent();

			FilterSetting = fs;

			BrightnessScrollBar->Minimum = -BrightnessScrollBar->Maximum;
			BrightnessScrollBar->Value   =  FilterSetting->brightness;
			ContrastScrollBar->Minimum   = -ContrastScrollBar->Maximum;
			ContrastScrollBar->Value     =  FilterSetting->contrast;
			GammaScrollBar->Minimum      = -GammaScrollBar->Maximum;
			GammaScrollBar->Value        =  FilterSetting->gamma;

			SWidth = width;
			SHeight = height;

			FilterCancelled = true;

			if (fs->channelcountin == 1)
			{
				PreviewBitmap = gcnew Bitmap(Preview->ClientSize.Width, Preview->ClientSize.Height, pFormat = PixelFormat::Format8bppIndexed);
				ColorPalette^ cPal = PreviewBitmap->Palette;
				for (int i = 0; i < 256; i++) cPal->Entries[i] = Color::FromArgb(i, i, i);
				PreviewBitmap->Palette = cPal;
				this->Width = 475;
			}
			else
			{
				PreviewBitmap = gcnew Bitmap(Preview->ClientSize.Width, Preview->ClientSize.Height, pFormat = PixelFormat::Format24bppRgb);
			}

			DestinationRect.X = (width < Preview->ClientSize.Width) ? ((Preview->ClientSize.Width - width) / 2) : 0;
			DestinationRect.Y = (height < Preview->ClientSize.Height) ? ((Preview->ClientSize.Height - height) / 2) : 0;
			DestinationRect.Width = (DestinationRect.X) ? width : Preview->ClientSize.Width;
			DestinationRect.Height = (DestinationRect.Y) ? height : Preview->ClientSize.Height;

			SourceRect.X = 0;
			SourceRect.Y = 0;
			SourceRect.Width = DestinationRect.Width;   // SHOULD BE EQUAL !!!
			SourceRect.Height = DestinationRect.Height; // SHOULD BE EQUAL !!!

			MouseButtonDown = false;

			unsigned int selector = FilterSetting->selector;

			CheckBoxBrightnessR->Checked = (selector & 0x100) != 0;
			CheckBoxBrightnessG->Checked = (selector & 0x080) != 0;
			CheckBoxBrightnessB->Checked = (selector & 0x040) != 0;
			CheckBoxContrastR->Checked   = (selector & 0x020) != 0;
			CheckBoxContrastG->Checked   = (selector & 0x010) != 0;
			CheckBoxContrastB->Checked   = (selector & 0x008) != 0;
			CheckBoxGammaR->Checked      = (selector & 0x004) != 0;
			CheckBoxGammaG->Checked      = (selector & 0x002) != 0;
			CheckBoxGammaB->Checked      = (selector & 0x001) != 0;

			init = 0;
			Scroll_Change(nullptr, nullptr);
		}

	protected:

		~SettingsForm()
		{
			if (components)
			{
				delete components;
			}
		}


	private: System::Windows::Forms::Button^  ButtonApply;
		System::Windows::Forms::Button^  ButtonCancel;
		System::Windows::Forms::PictureBox^  Preview;
		System::ComponentModel::IContainer^  components;
	

#pragma region Windows Form Designer generated code

		void InitializeComponent(void)
		{
			this->Preview = (gcnew System::Windows::Forms::PictureBox());
			this->ButtonApply = (gcnew System::Windows::Forms::Button());
			this->ButtonCancel = (gcnew System::Windows::Forms::Button());
			this->BrightnessScrollBar = (gcnew System::Windows::Forms::HScrollBar());
			this->ContrastScrollBar = (gcnew System::Windows::Forms::HScrollBar());
			this->GammaScrollBar = (gcnew System::Windows::Forms::HScrollBar());
			this->label1 = (gcnew System::Windows::Forms::Label());
			this->label2 = (gcnew System::Windows::Forms::Label());
			this->label3 = (gcnew System::Windows::Forms::Label());
			this->BrightnessLabel = (gcnew System::Windows::Forms::Label());
			this->ContrastLabel = (gcnew System::Windows::Forms::Label());
			this->GammaLabel = (gcnew System::Windows::Forms::Label());
			this->CheckBoxBrightnessR = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxBrightnessG = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxBrightnessB = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxContrastR = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxContrastG = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxContrastB = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxGammaR = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxGammaG = (gcnew System::Windows::Forms::CheckBox());
			this->CheckBoxGammaB = (gcnew System::Windows::Forms::CheckBox());
			this->label4 = (gcnew System::Windows::Forms::Label());
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Preview))->BeginInit();
			this->SuspendLayout();
			// 
			// Preview
			// 
			this->Preview->BorderStyle = System::Windows::Forms::BorderStyle::Fixed3D;
			this->Preview->Cursor = System::Windows::Forms::Cursors::NoMove2D;
			this->Preview->Location = System::Drawing::Point(12, 12);
			this->Preview->Name = L"Preview";
			this->Preview->Size = System::Drawing::Size(154, 154);
			this->Preview->TabIndex = 0;
			this->Preview->TabStop = false;
			this->Preview->MouseMove += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseMove);
			this->Preview->MouseDown += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseDown);
			this->Preview->Paint += gcnew System::Windows::Forms::PaintEventHandler(this, &SettingsForm::Preview_OnPaint);
			this->Preview->MouseUp += gcnew System::Windows::Forms::MouseEventHandler(this, &SettingsForm::Preview_MouseUp);
			// 
			// ButtonApply
			// 
			this->ButtonApply->Location = System::Drawing::Point(183, 143);
			this->ButtonApply->Name = L"ButtonApply";
			this->ButtonApply->Size = System::Drawing::Size(75, 23);
			this->ButtonApply->TabIndex = 2;
			this->ButtonApply->Text = L"Apply";
			this->ButtonApply->UseVisualStyleBackColor = true;
			this->ButtonApply->Click += gcnew System::EventHandler(this, &SettingsForm::ButtonApply_Click);
			// 
			// ButtonCancel
			// 
			this->ButtonCancel->Location = System::Drawing::Point(264, 143);
			this->ButtonCancel->Name = L"ButtonCancel";
			this->ButtonCancel->Size = System::Drawing::Size(75, 23);
			this->ButtonCancel->TabIndex = 3;
			this->ButtonCancel->Text = L"Cancel";
			this->ButtonCancel->UseVisualStyleBackColor = true;
			this->ButtonCancel->Click += gcnew System::EventHandler(this, &SettingsForm::ButtonCancel_Click);
			// 
			// BrightnessScrollBar
			// 
			this->BrightnessScrollBar->LargeChange = 1;
			this->BrightnessScrollBar->Location = System::Drawing::Point(183, 26);
			this->BrightnessScrollBar->Maximum = 255;
			this->BrightnessScrollBar->Minimum = 255;
			this->BrightnessScrollBar->Name = L"BrightnessScrollBar";
			this->BrightnessScrollBar->Size = System::Drawing::Size(250, 17);
			this->BrightnessScrollBar->TabIndex = 7;
			this->BrightnessScrollBar->Value = 255;
			this->BrightnessScrollBar->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &SettingsForm::Scroll_Change);
			// 
			// ContrastScrollBar
			// 
			this->ContrastScrollBar->LargeChange = 1;
			this->ContrastScrollBar->Location = System::Drawing::Point(183, 67);
			this->ContrastScrollBar->Maximum = 127;
			this->ContrastScrollBar->Minimum = 127;
			this->ContrastScrollBar->Name = L"ContrastScrollBar";
			this->ContrastScrollBar->Size = System::Drawing::Size(250, 17);
			this->ContrastScrollBar->TabIndex = 8;
			this->ContrastScrollBar->Value = 127;
			this->ContrastScrollBar->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &SettingsForm::Scroll_Change);
			// 
			// GammaScrollBar
			// 
			this->GammaScrollBar->LargeChange = 1;
			this->GammaScrollBar->Location = System::Drawing::Point(183, 107);
			this->GammaScrollBar->Minimum = 100;
			this->GammaScrollBar->Name = L"GammaScrollBar";
			this->GammaScrollBar->Size = System::Drawing::Size(250, 17);
			this->GammaScrollBar->TabIndex = 9;
			this->GammaScrollBar->Value = 100;
			this->GammaScrollBar->Scroll += gcnew System::Windows::Forms::ScrollEventHandler(this, &SettingsForm::Scroll_Change);
			// 
			// label1
			// 
			this->label1->AutoSize = true;
			this->label1->ForeColor = System::Drawing::Color::White;
			this->label1->Location = System::Drawing::Point(180, 12);
			this->label1->Name = L"label1";
			this->label1->Size = System::Drawing::Size(56, 13);
			this->label1->TabIndex = 10;
			this->label1->Text = L"Brightness";
			// 
			// label2
			// 
			this->label2->AutoSize = true;
			this->label2->ForeColor = System::Drawing::Color::White;
			this->label2->Location = System::Drawing::Point(180, 52);
			this->label2->Name = L"label2";
			this->label2->Size = System::Drawing::Size(46, 13);
			this->label2->TabIndex = 11;
			this->label2->Text = L"Contrast";
			// 
			// label3
			// 
			this->label3->AutoSize = true;
			this->label3->ForeColor = System::Drawing::Color::White;
			this->label3->Location = System::Drawing::Point(180, 92);
			this->label3->Name = L"label3";
			this->label3->Size = System::Drawing::Size(43, 13);
			this->label3->TabIndex = 12;
			this->label3->Text = L"Gamma";
			// 
			// BrightnessLabel
			// 
			this->BrightnessLabel->AutoSize = true;
			this->BrightnessLabel->ForeColor = System::Drawing::Color::White;
			this->BrightnessLabel->Location = System::Drawing::Point(436, 30);
			this->BrightnessLabel->Name = L"BrightnessLabel";
			this->BrightnessLabel->Size = System::Drawing::Size(21, 13);
			this->BrightnessLabel->TabIndex = 13;
			this->BrightnessLabel->Text = L"0%";
			// 
			// ContrastLabel
			// 
			this->ContrastLabel->AutoSize = true;
			this->ContrastLabel->ForeColor = System::Drawing::Color::White;
			this->ContrastLabel->Location = System::Drawing::Point(436, 71);
			this->ContrastLabel->Name = L"ContrastLabel";
			this->ContrastLabel->Size = System::Drawing::Size(21, 13);
			this->ContrastLabel->TabIndex = 14;
			this->ContrastLabel->Text = L"0%";
			// 
			// GammaLabel
			// 
			this->GammaLabel->AutoSize = true;
			this->GammaLabel->ForeColor = System::Drawing::Color::White;
			this->GammaLabel->Location = System::Drawing::Point(436, 111);
			this->GammaLabel->Name = L"GammaLabel";
			this->GammaLabel->Size = System::Drawing::Size(21, 13);
			this->GammaLabel->TabIndex = 15;
			this->GammaLabel->Text = L"0%";
			// 
			// CheckBoxBrightnessR
			// 
			this->CheckBoxBrightnessR->BackColor = System::Drawing::Color::Red;
			this->CheckBoxBrightnessR->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxBrightnessR->Checked = true;
			this->CheckBoxBrightnessR->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxBrightnessR->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxBrightnessR->Location = System::Drawing::Point(475, 26);
			this->CheckBoxBrightnessR->Name = L"CheckBoxBrightnessR";
			this->CheckBoxBrightnessR->Size = System::Drawing::Size(17, 17);
			this->CheckBoxBrightnessR->TabIndex = 16;
			this->CheckBoxBrightnessR->UseVisualStyleBackColor = false;
			this->CheckBoxBrightnessR->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxBrightnessG
			// 
			this->CheckBoxBrightnessG->BackColor = System::Drawing::Color::Lime;
			this->CheckBoxBrightnessG->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxBrightnessG->Checked = true;
			this->CheckBoxBrightnessG->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxBrightnessG->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxBrightnessG->Location = System::Drawing::Point(496, 26);
			this->CheckBoxBrightnessG->Name = L"CheckBoxBrightnessG";
			this->CheckBoxBrightnessG->Size = System::Drawing::Size(17, 17);
			this->CheckBoxBrightnessG->TabIndex = 17;
			this->CheckBoxBrightnessG->UseVisualStyleBackColor = false;
			this->CheckBoxBrightnessG->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxBrightnessB
			// 
			this->CheckBoxBrightnessB->BackColor = System::Drawing::Color::Blue;
			this->CheckBoxBrightnessB->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxBrightnessB->Checked = true;
			this->CheckBoxBrightnessB->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxBrightnessB->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxBrightnessB->Location = System::Drawing::Point(517, 26);
			this->CheckBoxBrightnessB->Name = L"CheckBoxBrightnessB";
			this->CheckBoxBrightnessB->Size = System::Drawing::Size(17, 17);
			this->CheckBoxBrightnessB->TabIndex = 18;
			this->CheckBoxBrightnessB->UseVisualStyleBackColor = false;
			this->CheckBoxBrightnessB->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxContrastR
			// 
			this->CheckBoxContrastR->BackColor = System::Drawing::Color::Red;
			this->CheckBoxContrastR->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxContrastR->Checked = true;
			this->CheckBoxContrastR->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxContrastR->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxContrastR->Location = System::Drawing::Point(475, 67);
			this->CheckBoxContrastR->Name = L"CheckBoxContrastR";
			this->CheckBoxContrastR->Size = System::Drawing::Size(17, 17);
			this->CheckBoxContrastR->TabIndex = 19;
			this->CheckBoxContrastR->UseVisualStyleBackColor = false;
			this->CheckBoxContrastR->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxContrastG
			// 
			this->CheckBoxContrastG->BackColor = System::Drawing::Color::Lime;
			this->CheckBoxContrastG->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxContrastG->Checked = true;
			this->CheckBoxContrastG->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxContrastG->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxContrastG->Location = System::Drawing::Point(496, 67);
			this->CheckBoxContrastG->Name = L"CheckBoxContrastG";
			this->CheckBoxContrastG->Size = System::Drawing::Size(17, 17);
			this->CheckBoxContrastG->TabIndex = 20;
			this->CheckBoxContrastG->UseVisualStyleBackColor = false;
			this->CheckBoxContrastG->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxContrastB
			// 
			this->CheckBoxContrastB->BackColor = System::Drawing::Color::Blue;
			this->CheckBoxContrastB->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxContrastB->Checked = true;
			this->CheckBoxContrastB->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxContrastB->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxContrastB->Location = System::Drawing::Point(517, 67);
			this->CheckBoxContrastB->Name = L"CheckBoxContrastB";
			this->CheckBoxContrastB->Size = System::Drawing::Size(17, 17);
			this->CheckBoxContrastB->TabIndex = 21;
			this->CheckBoxContrastB->UseVisualStyleBackColor = false;
			this->CheckBoxContrastB->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxGammaR
			// 
			this->CheckBoxGammaR->BackColor = System::Drawing::Color::Red;
			this->CheckBoxGammaR->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxGammaR->Checked = true;
			this->CheckBoxGammaR->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxGammaR->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxGammaR->Location = System::Drawing::Point(475, 107);
			this->CheckBoxGammaR->Name = L"CheckBoxGammaR";
			this->CheckBoxGammaR->Size = System::Drawing::Size(17, 17);
			this->CheckBoxGammaR->TabIndex = 22;
			this->CheckBoxGammaR->UseVisualStyleBackColor = false;
			this->CheckBoxGammaR->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxGammaG
			// 
			this->CheckBoxGammaG->BackColor = System::Drawing::Color::Lime;
			this->CheckBoxGammaG->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxGammaG->Checked = true;
			this->CheckBoxGammaG->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxGammaG->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxGammaG->Location = System::Drawing::Point(496, 107);
			this->CheckBoxGammaG->Name = L"CheckBoxGammaG";
			this->CheckBoxGammaG->Size = System::Drawing::Size(17, 17);
			this->CheckBoxGammaG->TabIndex = 23;
			this->CheckBoxGammaG->UseVisualStyleBackColor = false;
			this->CheckBoxGammaG->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// CheckBoxGammaB
			// 
			this->CheckBoxGammaB->BackColor = System::Drawing::Color::Blue;
			this->CheckBoxGammaB->CheckAlign = System::Drawing::ContentAlignment::MiddleCenter;
			this->CheckBoxGammaB->Checked = true;
			this->CheckBoxGammaB->CheckState = System::Windows::Forms::CheckState::Checked;
			this->CheckBoxGammaB->FlatStyle = System::Windows::Forms::FlatStyle::Flat;
			this->CheckBoxGammaB->Location = System::Drawing::Point(517, 107);
			this->CheckBoxGammaB->Name = L"CheckBoxGammaB";
			this->CheckBoxGammaB->Size = System::Drawing::Size(17, 17);
			this->CheckBoxGammaB->TabIndex = 24;
			this->CheckBoxGammaB->UseVisualStyleBackColor = false;
			this->CheckBoxGammaB->CheckedChanged += gcnew System::EventHandler(this, &SettingsForm::CheckBoxBCG_CheckedChanged);
			// 
			// label4
			// 
			this->label4->AutoSize = true;
			this->label4->ForeColor = System::Drawing::Color::White;
			this->label4->Location = System::Drawing::Point(472, 10);
			this->label4->Name = L"label4";
			this->label4->Size = System::Drawing::Size(60, 13);
			this->label4->TabIndex = 25;
			this->label4->Text = L" R     G    B";
			// 
			// SettingsForm
			// 
			this->AutoScaleDimensions = System::Drawing::SizeF(6, 13);
			this->AutoScaleMode = System::Windows::Forms::AutoScaleMode::Font;
			this->BackColor = System::Drawing::Color::SteelBlue;
			this->ClientSize = System::Drawing::Size(554, 178);
			this->Controls->Add(this->label4);
			this->Controls->Add(this->CheckBoxGammaB);
			this->Controls->Add(this->CheckBoxGammaG);
			this->Controls->Add(this->CheckBoxGammaR);
			this->Controls->Add(this->CheckBoxContrastB);
			this->Controls->Add(this->CheckBoxContrastG);
			this->Controls->Add(this->CheckBoxContrastR);
			this->Controls->Add(this->CheckBoxBrightnessB);
			this->Controls->Add(this->CheckBoxBrightnessG);
			this->Controls->Add(this->CheckBoxBrightnessR);
			this->Controls->Add(this->GammaLabel);
			this->Controls->Add(this->ContrastLabel);
			this->Controls->Add(this->BrightnessLabel);
			this->Controls->Add(this->label3);
			this->Controls->Add(this->label2);
			this->Controls->Add(this->label1);
			this->Controls->Add(this->GammaScrollBar);
			this->Controls->Add(this->ContrastScrollBar);
			this->Controls->Add(this->BrightnessScrollBar);
			this->Controls->Add(this->ButtonCancel);
			this->Controls->Add(this->ButtonApply);
			this->Controls->Add(this->Preview);
			this->FormBorderStyle = System::Windows::Forms::FormBorderStyle::FixedSingle;
			this->MaximizeBox = false;
			this->MinimizeBox = false;
			this->Name = L"SettingsForm";
			this->ShowIcon = false;
			this->ShowInTaskbar = false;
			this->StartPosition = System::Windows::Forms::FormStartPosition::CenterScreen;
			this->Text = L"Brightness/Contrast/Gamma Adjust";
			(cli::safe_cast<System::ComponentModel::ISupportInitialize^  >(this->Preview))->EndInit();
			this->ResumeLayout(false);
			this->PerformLayout();

		}
#pragma endregion

	public:
		void RefreshPreview(bool applyfilter)
		{
			BitmapData^ bd = PreviewBitmap->LockBits( Rectangle(0, 0, PreviewBitmap->Width, PreviewBitmap->Height), ImageLockMode::ReadWrite, pFormat);

			FilterSetting->strideout = bd->Stride;
			FilterSetting->dataout = (Byte*)(void*)bd->Scan0;

			if (applyfilter)
				FilterSpace::ImageFilter::FilterCore(FilterSetting, SourceRect.X, SourceRect.Y, SourceRect.Width, SourceRect.Height);
			else
				FilterSpace::ImageFilter::ClipImage( FilterSetting, SourceRect.X, SourceRect.Y, SourceRect.Width, SourceRect.Height);

			PreviewBitmap->UnlockBits(bd);
			this->Refresh();
		}

	private: 
		void ButtonApply_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			FilterCancelled = false;
			this->Close();
		}

		void ButtonCancel_Click(System::Object^  sender, System::EventArgs^  e) 
		{
			this->Close();
		}

		void Preview_MouseDown(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (e->Button == System::Windows::Forms::MouseButtons::Left) 
			{
				MouseButtonDown = true;
				LastX = e->X;
				LastY = e->Y;
				RefreshPreview(false);
			}
		}

		void Preview_MouseMove(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (!MouseButtonDown) return;
			if (!DestinationRect.X)
			{
				SourceRect.X -= e->X - LastX;
				if (SourceRect.X < 0) SourceRect.X = 0;
				if (SourceRect.X >= SWidth - DestinationRect.Width) SourceRect.X = SWidth - DestinationRect.Width - 1;
			}
			if (!DestinationRect.Y)
			{
				SourceRect.Y -= e->Y - LastY;
				if (SourceRect.Y < 0) SourceRect.Y = 0;
				if (SourceRect.Y >= SHeight - DestinationRect.Height) SourceRect.Y = SHeight - DestinationRect.Height - 1;
			}
			RefreshPreview(false);
			LastX = e->X;
			LastY = e->Y;
		}

		void Preview_MouseUp(System::Object^  sender, System::Windows::Forms::MouseEventArgs^  e) 
		{
			if (e->Button == System::Windows::Forms::MouseButtons::Left) MouseButtonDown = false;
			if (!MouseButtonDown) RefreshPreview(true);
		}

		void Preview_OnPaint(System::Object^  sender, System::Windows::Forms::PaintEventArgs^  e) 
		{
			Graphics^ g = e->Graphics;
			Rectangle r = Rectangle(0, 0, DestinationRect.Width, DestinationRect.Height);
			g->DrawImage(PreviewBitmap, DestinationRect, r, GraphicsUnit::Pixel);
		}

		void Scroll_Change(System::Object^  sender, System::Windows::Forms::ScrollEventArgs^  e) 
		{
			if (init) return;

			unsigned int selector = 0;
			selector |= (CheckBoxBrightnessR->Checked) ? 0x100 : 0;
			selector |= (CheckBoxBrightnessG->Checked) ? 0x080 : 0;
			selector |= (CheckBoxBrightnessB->Checked) ? 0x040 : 0;
			selector |= (CheckBoxContrastR->Checked)   ? 0x020 : 0;
			selector |= (CheckBoxContrastG->Checked)   ? 0x010 : 0;
			selector |= (CheckBoxContrastB->Checked)   ? 0x008 : 0;
			selector |= (CheckBoxGammaR->Checked)      ? 0x004 : 0;
			selector |= (CheckBoxGammaG->Checked)      ? 0x002 : 0;
			selector |= (CheckBoxGammaB->Checked)      ? 0x001 : 0;
			FilterSetting->brightness = BrightnessScrollBar->Value;
			FilterSetting->contrast   = ContrastScrollBar->Value;
			FilterSetting->gamma      = GammaScrollBar->Value;
			FilterSetting->selector   = selector;

			BrightnessLabel->Text = ((BrightnessScrollBar->Value * 100) / 255).ToString() + "%";
			ContrastLabel->Text = ((ContrastScrollBar->Value * 100) / 127).ToString() + "%";
			GammaLabel->Text = GammaScrollBar->Value.ToString() + "%";
			RefreshPreview(true);
		}

		void CheckBoxBCG_CheckedChanged(System::Object^  sender, System::EventArgs^  e) { Scroll_Change(nullptr, nullptr); }
};
}
